@@ -778,7 +778,15 @@ macro_rules! impl_fetch_for_tuples {
 
     ($head:ident $(,$tail:ident)*) => {
         impl_fetch_for_tuples!($($tail),*);
-        impl_fetch_for_tuples!(@impl $head $(,$tail)*);
+        impl_fetch_for_tuples!(@rev $head $(,$tail)*;);
+    };
+
+    (@rev ; $($rev:ident),*) => {
+        impl_fetch_for_tuples!(@impl $($rev),*);
+    };
+
+    (@rev $head:ident $(,$tail:ident)*; $($rev:ident),*) => {
+        impl_fetch_for_tuples!(@rev $($tail),*; $head $(,$rev)*);
     };
 
     (@impl $($types:ident),+) => {
@@ -840,7 +848,7 @@ macro_rules! impl_fetch_for_tuples {
     };
 }
 
-impl_fetch_for_tuples!(A, B, C, D, E, F, G, H, I, J);
+impl_fetch_for_tuples!(J, I, H, G, F, E, D, C, B, A);
 
 #[cfg(test)]
 mod tests {
