@@ -79,7 +79,7 @@ impl World {
         let ent = Entity { id, gen: meta.gen };
 
         unsafe {
-            self.archetypes[0].pointer::<Entity>(meta.idx).write(ent);
+            self.archetypes[0].get_raw::<Entity>(meta.idx).write(ent);
         }
 
         ent
@@ -131,7 +131,7 @@ impl World {
     ) {
         unsafe {
             if archetype.free::<DROP>(idx) {
-                let swapped_ent = archetype.pointer::<Entity>(idx).read();
+                let swapped_ent = archetype.get_raw::<Entity>(idx).read();
 
                 entities[swapped_ent.id as usize].idx = idx;
             }
@@ -392,10 +392,13 @@ impl World {
     }
 
     unsafe fn move_(&mut self, id: u32, old_ty: u16, new_ty: u16, old_idx: u32) -> u32 {
+        debug_assert!(self.archetypes.len() > old_ty as usize);
+        debug_assert!(self.archetypes.len() > new_ty as usize);
         debug_assert_ne!(old_ty, new_ty);
 
-        let old_archetype = &mut *self.archetypes.as_mut_ptr().add(old_ty as usize);
-        let new_archetype = &mut *self.archetypes.as_mut_ptr().add(new_ty as usize);
+        let archetypes = self.archetypes.as_mut_ptr();
+        let old_archetype = &mut *archetypes.add(old_ty as usize);
+        let new_archetype = &mut *archetypes.add(new_ty as usize);
 
         let new_idx = new_archetype.alloc();
 
@@ -478,7 +481,7 @@ impl World {
 
         unsafe {
             other.archetypes[new_meta.ty as usize]
-                .pointer::<Entity>(new_meta.idx)
+                .get_raw::<Entity>(new_meta.idx)
                 .write(ent);
         }
 
@@ -679,7 +682,15 @@ macro_rules! impl_bundle_for_tuples {
 
     ($head:ident $(,$tail:ident)*) => {
         impl_bundle_for_tuples!($($tail),*);
-        impl_bundle_for_tuples!(@impl $head $(,$tail)*);
+        impl_bundle_for_tuples!(@rev $head $(,$tail)*;);
+    };
+
+    (@rev ; $($rev:ident),*) => {
+        impl_bundle_for_tuples!(@impl $($rev),*);
+    };
+
+    (@rev $head:ident $(,$tail:ident)*; $($rev:ident),*) => {
+        impl_bundle_for_tuples!(@rev $($tail),*; $head $(,$rev)*);
     };
 
     (@impl $($types:ident),+) => {
@@ -708,19 +719,19 @@ macro_rules! impl_bundle_for_tuples {
             #[allow(non_snake_case)]
             unsafe fn write(self, archetype: &mut Archetype, idx: u32) {
                 let ($($types,)+) = self;
-                $(archetype.pointer::<$types>(idx).write($types);)+
+                $(archetype.get_raw::<$types>(idx).write($types);)+
             }
 
             #[allow(non_snake_case)]
             unsafe fn read(archetype: &mut Archetype, idx: u32) -> Self {
-                $(let $types = archetype.pointer::<$types>(idx).read();)+
+                $(let $types = archetype.get_raw::<$types>(idx).read();)+
                 ($($types,)+)
             }
         }
     };
 }
 
-impl_bundle_for_tuples!(A, B, C, D, E, F, G, H, I, J);
+impl_bundle_for_tuples!(J, I, H, G, F, E, D, C, B, A);
 
 type IndexTypeIdMap<V> = HashMap<(u16, TypeId), V, BuildHasherDefault<IndexTypeIdHasher>>;
 
@@ -833,7 +844,7 @@ mod tests {
     }
 
     #[test]
-    fn adding_component_creates_archetype() {
+    fn inserting_component_creates_archetype() {
         let mut world = World::new();
 
         assert_eq!(world.entities.len(), 0);
@@ -941,9 +952,17 @@ mod tests {
     #[test]
     fn trival_exchange_does_not_create_aliasing_unique_references() {
         let mut world = World::new();
+
         let ent = world.alloc();
         world.insert(ent, (true,));
+
         world.exchange::<(bool,), _>(ent, (false,)).unwrap();
+
+        assert_eq!(world.exchange_map.len(), 1);
+        assert_eq!(
+            world.exchange_map[&(1, TypeId::of::<((bool,), (bool,))>())],
+            1
+        );
     }
 
     #[test]
